# -*- coding: utf-8 -*-

import os
import pprint
import pickle
from collection import Collection
from display import Display

class Coach:
    """ The coach supervises the selection of exercises. """

    def __init__(self):
        self._selected_tree_position = None
        self._selected_object = None
        self._current_exercise = None

    def register(self, sess, dis, exstore, learner):
        self._sess = sess
        self._dis = dis
        self._exstore = exstore
        self._learner = learner

    def request_exercise(self):
        # Called to load the exstore into the tree
        # This preliminary version just lets the Learner select.
        self._exstore.offer_coll_to_learner()

    def get_selected_object(self, pos):
        # Returns the object (task, collection) of a given tree position
        self._selected_tree_position = pos
        self._selected_object = self._exstore._root.get_object_by_tupel(pos)
        return self._selected_object

    def switch_to_settings(self):
        # Load selected exercise
        self._current_exercise = self._exstore.get_exercise(self._selected_object)
        # Show settings
        self._dis.offer_setting(self._current_exercise)

    def settings_done(self, sett):
        # Load selected exercise
        self._current_exercise.update_setting(sett)
        # Start calculations with user defined settings
        self._learner.start_calcs(self._current_exercise)

    def exercise_selected(self):
        # Reload selected exercise (without settings manipulation)
        self._selected_object = self._exstore._root.get_object_by_tupel(self._selected_tree_position)
        self._current_exercise = self._exstore.get_exercise(self._selected_object)
        # Start calculations
        self._learner.start_calcs(self._current_exercise)

